"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRuntime = void 0;
const web_runtime_1 = require("./web-runtime.js");
const node_fs_1 = require("node:fs");
function getRuntime() {
    const runtime = (0, web_runtime_1.getRuntime)();
    function isFsReadStream(value) {
        return value instanceof node_fs_1.ReadStream;
    }
    return { ...runtime, isFsReadStream };
}
exports.getRuntime = getRuntime;//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/7772f905dc6ba5632ae304533f286ac049f59543/node_modules/@anthropic-ai/sdk/_shims/bun-runtime.js.map